/*
 * Copyright (c) 2008 Nokia Corporation
 * Contact: integration@maemo.org
 *
 * Authors: Hallyson Melo <hallyson.melo@indt.org.br>
 *          Kyller Gorgonio <kyllercg@gmail.com>
 *          Walter Guerra <wguerra@ee.ufcg.edu.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 * USA.
 */

#include <hildon/hildon.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>

#include "nm_main_dialog.h"
#include "nm_conn_widget.h"
#include "nm_bt_setup.h"
#include "nm_bt_utils.h"
#include "nm_usb_setup.h"
#include "nm_usb_utils.h"
#include "nm_wlan_setup.h"
#include "nm_wlan_utils.h"
#include "nm_settings.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define NM_CONN_TYPE_USB               0
#define NM_CONN_TYPE_BT                1
#define NM_CONN_TYPE_WLAN              2
#define NM_AVAILABLE_CONN_TYPES        3

enum
{
   CONNECTION_NAME_COLUMN,
   CONNECTION_WIDGET_COLUMN,
   N_COLUMNS
};

static void nm_main_dialog_class_init (NmMainDialogClass *);
static void nm_main_dialog_init       (NmMainDialog *);
static void nm_main_dialog_dispose    (GObject *);
static void nm_main_dialog_finalize   (GObject *);

static void nm_main_dialog_create_tree_view (NmMainDialog *);
static void nm_main_dialog_add_widget (NmMainDialog *, gchar *, NmConnWidget *,
                                        gboolean);
static void nm_main_dialog_run_ok (NmMainDialog *);
static void nm_main_dialog_selection_changed (GtkTreeSelection *, gpointer);
static NmConnWidget* nm_main_dialog_get_selected (NmMainDialog *);
static void nm_main_dialog_settings_changed (gpointer, gpointer);
static void nm_main_dialog_setup_basic (NmMainDialog *);
static void nm_main_dialog_setup_advanced (NmMainDialog *);
static void nm_main_dialog_switch_to_advanced (NmMainDialog *);
static void nm_main_dialog_switch_to_basic (NmMainDialog *);
static gboolean nm_main_dialog_usb_apply (NmMainDialog *);
static gboolean nm_main_dialog_bt_apply (NmMainDialog *);
static gboolean nm_main_dialog_wlan_apply (NmMainDialog *);

typedef struct _NmMainDialogPrivate NmMainDialogPrivate;

struct _NmMainDialogPrivate
{
    gint current_mode;
    NmSettings *settings;
    gint basic_length;
    gint basic_height;
    gint basic_x;
    gint basic_y;
    gint selected_checkbox;

    GtkWidget *treeview;
    GtkTreeStore *store;

    GtkWidget *btsetup;
    GtkWidget *usbsetup;
    GtkWidget *wlansetup;

    GtkWidget *ok_button;
    GtkWidget *apply_button;
    GtkWidget *cancel_button;
    GtkWidget *advanced_button;
    GtkWidget *basic_button;
    GtkWidget *reset_button;

    GtkWidget *basic_hbox;
    GtkWidget *advanced_hbox;

    GtkWidget *check_basic_usb;
    GtkWidget *check_basic_bt;
    GtkWidget *check_basic_wlan;
};

#define NM_MAIN_DIALOG_GET_PRIVATE(o) \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_MAIN_DIALOG_TYPE, NmMainDialogPrivate))

G_DEFINE_TYPE (NmMainDialog, nm_main_dialog, GTK_TYPE_DIALOG)

static void
nm_main_dialog_class_init (NmMainDialogClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    g_type_class_add_private (klass, sizeof (NmMainDialogPrivate));

    object_class->dispose = nm_main_dialog_dispose;
    object_class->finalize = nm_main_dialog_finalize;
}

static void
nm_main_dialog_init (NmMainDialog *self)
{
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    gtk_window_set_title (GTK_WINDOW(self), "Network Manager" );

    nm_main_dialog_setup_advanced (self);
    nm_main_dialog_setup_basic (self);

    priv->settings = nm_settings_new ();
    if (!nm_settings_start (priv->settings)) {

        hildon_banner_show_information(GTK_WIDGET (self), NULL,
                                        "Could not read config file!\n\
                                         Settings defaults...");
	// FIXME
        g_usleep(1000000);
        nm_settings_set_defaults(priv->settings);
    }

    /* Add the buttons */
    priv->ok_button = gtk_dialog_add_button (GTK_DIALOG (self),
                                              _("Apply"),
                                              GTK_RESPONSE_OK);

    priv->advanced_button = gtk_dialog_add_button (GTK_DIALOG (self),
                                                    _("Advanced"),
                                                    NM_RESPONSE_ADVANCED);          

    gtk_button_set_focus_on_click(GTK_BUTTON(priv->advanced_button), FALSE);


    priv->basic_button = gtk_dialog_add_button (GTK_DIALOG (self),
                                                 _("Basic"),
                                                 NM_RESPONSE_BASIC);

    priv->cancel_button = gtk_dialog_add_button (GTK_DIALOG (self),
                                                  _("Close"),
                                                  GTK_RESPONSE_CLOSE);

    gtk_widget_set_sensitive (priv->basic_button, TRUE);

    /* Fixme: Conclude reset implementation. Something is still missing */
    /*
    priv->reset_button = gtk_dialog_add_button (GTK_DIALOG (self),
                                                 _("Reset"),
                                                 NM_RESPONSE_RESET);
    */

    // Set the default checkbox to 0
    priv->selected_checkbox = 0;

    gtk_widget_hide (priv->basic_button);

    // FIXME: remove this later
    priv->apply_button = priv->advanced_button;

    gtk_window_get_size (GTK_WINDOW(self), &(priv->basic_length),
                          &(priv->basic_height));
    gtk_window_get_position (GTK_WINDOW(self), &(priv->basic_x),
                              &(priv->basic_y));
}

static void
nm_main_dialog_dispose (GObject *object)
{
    G_OBJECT_CLASS (nm_main_dialog_parent_class)->dispose (object);
}

static void
nm_main_dialog_finalize (GObject *object)
{
    G_OBJECT_CLASS (nm_main_dialog_parent_class)->finalize (object);
}

GtkWidget*
nm_main_dialog_new ()
{
    return g_object_new (NM_MAIN_DIALOG_TYPE, NULL);
}

static void
nm_main_dialog_create_tree_view (NmMainDialog *self)
{
    GtkTreeStore *store;
    GtkCellRenderer *renderer;
    GtkTreeSelection *selection;
    GtkTreeViewColumn *column;
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    store = gtk_tree_store_new (N_COLUMNS,
                                 G_TYPE_STRING,
                                 NM_CONN_WIDGET_TYPE);

    priv->treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));

    /* Creates the render */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Connection", renderer,
                                                        "text",
                                                        CONNECTION_NAME_COLUMN,
                                                        NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);

    /* Setup the selection handler */
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
    g_signal_connect (G_OBJECT (selection), "changed",
                       G_CALLBACK (nm_main_dialog_selection_changed),
                       self);
}

static void 
nm_main_dialog_add_widget (NmMainDialog *self, gchar *name,
                            NmConnWidget *widget, gboolean is_default)
{
    GtkTreeModel *model;
    NmMainDialogPrivate *priv;
    GtkTreeIter iter;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));

    /* Append the network item */
    gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
    gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
                         CONNECTION_NAME_COLUMN, name,
                         CONNECTION_WIDGET_COLUMN, widget, -1);

    if (is_default) {

        gtk_tree_selection_select_iter
            (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
                                           &iter);
    }
}

void
nm_main_dialog_run (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);
    gint ret;
    gboolean done = FALSE;

    while (!done) {

        ret = gtk_dialog_run (GTK_DIALOG (self));

        switch (ret) {

            case GTK_RESPONSE_CLOSE:

                done = TRUE;
                break;

            case GTK_RESPONSE_OK:

                nm_main_dialog_run_ok (self);
                break;

            case NM_RESPONSE_ADVANCED:

                nm_main_dialog_switch_to_advanced (self);
                break;

            case NM_RESPONSE_BASIC:

                nm_main_dialog_switch_to_basic (self);
                gtk_window_resize (GTK_WINDOW(self), priv->basic_length,
                                    priv->basic_height);
                gtk_window_move (GTK_WINDOW(self), priv->basic_x,
                                  priv->basic_y);
                break;

            default:

                break;
        }
    }
}

static void
nm_main_dialog_run_ok (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);
    NmConnWidget *conn_widget = NULL;

    if (priv->current_mode == NM_ADVANCED_MODE) {

        conn_widget = nm_main_dialog_get_selected (self);

        if (!nm_conn_widget_apply_changes (conn_widget)) {

            hildon_banner_show_information(GTK_WIDGET (self), NULL,
                                            "Connection settings error!");
        }
    } else if (priv->current_mode == NM_BASIC_MODE) {

        if (!nm_main_dialog_usb_apply (self)) {

            hildon_banner_show_information(GTK_WIDGET (self), NULL,
                                            "USB settings error!");
        }

        if (!nm_main_dialog_bt_apply (self)) {

            hildon_banner_show_information(GTK_WIDGET (self), NULL,
                                            "Bluetooth settings error!");
        }

        if (!nm_main_dialog_wlan_apply (self)) {

            hildon_banner_show_information(GTK_WIDGET (self), NULL,
                                            "WLAN settings error!");
        }
    }
}

static void
nm_main_dialog_selection_changed (GtkTreeSelection *new, gpointer data)
{
    NmMainDialogPrivate *priv;
    NmConnWidget *conn_widget;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (data);

    conn_widget = nm_main_dialog_get_selected (NM_MAIN_DIALOG (data));

    // FIXME this shown/hide panels need to be changed for a clever code
    if (conn_widget) {
        if (conn_widget == NM_CONN_WIDGET (priv->usbsetup)) {

            // hide BLUETOOTH and WLAN panels
            gtk_widget_hide (GTK_WIDGET (priv->btsetup));
            gtk_widget_hide (GTK_WIDGET (priv->wlansetup));

            // show USB panel
            gtk_widget_show (GTK_WIDGET (priv->usbsetup));
            
            // Set usb as the default panel
            priv->selected_checkbox = 0;
        } else if (conn_widget == NM_CONN_WIDGET (priv->btsetup)) {

            // hide USB and WLAN panels
            gtk_widget_hide (GTK_WIDGET (priv->usbsetup));
            gtk_widget_hide (GTK_WIDGET (priv->wlansetup));

            // show BLUETOOTH panel
            gtk_widget_show (GTK_WIDGET (priv->btsetup));
            
            // Set usb as the default panel
            priv->selected_checkbox = 1;
        } else if (conn_widget == NM_CONN_WIDGET (priv->wlansetup)) {

            // hide BLUETOOTH and USB panels
            gtk_widget_hide (GTK_WIDGET (priv->btsetup));
            gtk_widget_hide (GTK_WIDGET (priv->usbsetup));

            // show WLAN panel
            gtk_widget_show (GTK_WIDGET (priv->wlansetup));
            
            // Set wlan as the default panel
            priv->selected_checkbox = 2;
        }
        g_object_unref (conn_widget);
    }
}

static NmConnWidget*
nm_main_dialog_get_selected (NmMainDialog *self)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    NmConnWidget *conn_widget = NULL;
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    if (priv->current_mode == NM_ADVANCED_MODE) {

        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));

        if (gtk_tree_selection_get_selected (selection, &model, &iter)) {

            gtk_tree_model_get (model, &iter, CONNECTION_WIDGET_COLUMN,
                                 &conn_widget, -1);
        }
    }

    return conn_widget;
}

static void
nm_main_dialog_settings_changed (gpointer instance, gpointer data)
{
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (NM_MAIN_DIALOG (data));

    gtk_widget_set_sensitive (priv->ok_button, TRUE);
    gtk_widget_set_sensitive (priv->apply_button, TRUE);
}

static void
nm_main_dialog_setup_basic (NmMainDialog *self)
{
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    priv->basic_hbox = gtk_hbox_new (FALSE, 3);

    priv->check_basic_usb = gtk_check_button_new_with_label ("USB");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->check_basic_usb),
                                   usb_setup_check_enabled (NM_USB_SETUP(priv->usbsetup)));

    priv->check_basic_bt = gtk_check_button_new_with_label ("Bluetooth");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->check_basic_bt),
                                   bt_setup_check_enabled (NM_BT_SETUP(priv->btsetup)));

    priv->check_basic_wlan = gtk_check_button_new_with_label ("Wlan");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(priv->check_basic_wlan),
                                   wlan_setup_check_enabled (NM_WLAN_SETUP(priv->wlansetup)));

    gtk_box_pack_start (GTK_BOX(priv->basic_hbox), priv->check_basic_usb, FALSE, TRUE, 1);
    gtk_box_pack_start (GTK_BOX(priv->basic_hbox), priv->check_basic_bt, FALSE, TRUE, 1);
    gtk_box_pack_start (GTK_BOX(priv->basic_hbox), priv->check_basic_wlan, FALSE, TRUE, 1);

    gtk_widget_show_all (priv->basic_hbox);

    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (self)->vbox), priv->basic_hbox,
                         TRUE, TRUE, 6);

    priv->current_mode = NM_BASIC_MODE;
}

static void
nm_main_dialog_setup_advanced (NmMainDialog *self)
{
    GtkWidget *frame_tree;
    NmMainDialogPrivate *priv;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    priv->advanced_hbox = gtk_hbox_new (FALSE, 3);

    /* Creates the connection type area (frame) */
    frame_tree = gtk_frame_new (" Connection type ");
    nm_main_dialog_create_tree_view (self);

    priv->usbsetup = nm_usb_setup_new ();
    nm_main_dialog_add_widget (self, "USB Network",
                                NM_CONN_WIDGET (priv->usbsetup), TRUE);
    g_signal_connect (G_OBJECT (priv->usbsetup),
                       NM_CONN_WIDGET_SETTINGS_CHANGED,
                       G_CALLBACK (nm_main_dialog_settings_changed), self);

    priv->btsetup = nm_bt_setup_new ();
    nm_main_dialog_add_widget (self, "Bluetooth",
                                NM_CONN_WIDGET (priv->btsetup), FALSE);
    g_signal_connect (G_OBJECT (priv->btsetup),
                       NM_CONN_WIDGET_SETTINGS_CHANGED,
                       G_CALLBACK (nm_main_dialog_settings_changed), self);

    priv->wlansetup = nm_wlan_setup_new ();
    nm_main_dialog_add_widget (self, "Wlan", NM_CONN_WIDGET (priv->wlansetup),
                                FALSE);
    g_signal_connect (G_OBJECT (priv->wlansetup),
                       NM_CONN_WIDGET_SETTINGS_CHANGED,
                       G_CALLBACK (nm_main_dialog_settings_changed), self);

    gtk_container_add (GTK_CONTAINER (frame_tree), priv->treeview);
    gtk_box_pack_start (GTK_BOX (priv->advanced_hbox), frame_tree,
                         TRUE, TRUE, 0);

    gtk_box_pack_start (GTK_BOX (priv->advanced_hbox), priv->usbsetup,
                         FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (priv->advanced_hbox), priv->btsetup,
                         FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (priv->advanced_hbox), priv->wlansetup,
                         FALSE, TRUE, 0);

    /* Add the advanced_hbox to the vbox */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (self)->vbox), priv->advanced_hbox,
                         TRUE, TRUE, 6);
}

static void
nm_main_dialog_switch_to_advanced (NmMainDialog *self)
{
    NmMainDialogPrivate *priv;
    NmConnWidget *conn_widget;
    GtkTreeIter iter;
    GtkTreeModel *model;

    priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    
    // TODO
    gtk_tree_selection_unselect_all(gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)));
    // TODO END
    
    if ( gtk_widget_is_focus(GTK_WIDGET (priv->check_basic_usb)) ) {
        priv->selected_checkbox = 0;
    }
    else if ( gtk_widget_is_focus(GTK_WIDGET (priv->check_basic_bt)) ) {
        priv->selected_checkbox = 1;
    }
    else if ( gtk_widget_is_focus(GTK_WIDGET (priv->check_basic_wlan)) ) {
        priv->selected_checkbox = 2;
    }
    
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
    gtk_tree_model_iter_nth_child(model, &iter, NULL, priv->selected_checkbox);
    gtk_tree_selection_select_iter
        (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
                                       &iter);        
    
    

    priv->current_mode = NM_ADVANCED_MODE;

    conn_widget = nm_main_dialog_get_selected (self);

    // hide/show basic/advanced hboxes
    gtk_widget_hide (GTK_WIDGET (priv->basic_hbox));
    gtk_widget_show_all (GTK_WIDGET (priv->advanced_hbox));

    // hide/show advanced/basic buttons
    gtk_widget_hide (GTK_WIDGET (priv->advanced_button));
    gtk_widget_show (GTK_WIDGET (priv->basic_button));

    // check if has been enabled on basic mode
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->check_basic_usb)) ) {
        nm_usb_setup_enable(NM_USB_SETUP(priv->usbsetup), TRUE);
    }
    
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->check_basic_bt)) ) {
        nm_bt_setup_enable(NM_BT_SETUP(priv->btsetup), TRUE);
    }
    
    if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->check_basic_wlan)) ) {
        nm_wlan_setup_enable(NM_WLAN_SETUP(priv->wlansetup), TRUE);
    }    

    if (conn_widget == NM_CONN_WIDGET (priv->usbsetup)) {

        gtk_widget_hide (GTK_WIDGET (priv->btsetup));
        gtk_widget_hide (GTK_WIDGET (priv->wlansetup));
    } else if (conn_widget == NM_CONN_WIDGET (priv->btsetup)) {

        gtk_widget_hide (GTK_WIDGET (priv->usbsetup));
        gtk_widget_hide (GTK_WIDGET (priv->wlansetup));
    } else if (conn_widget == NM_CONN_WIDGET (priv->wlansetup)) {

        gtk_widget_hide (GTK_WIDGET (priv->usbsetup));
        gtk_widget_hide (GTK_WIDGET (priv->btsetup));
    }
}

static void
nm_main_dialog_switch_to_basic (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    priv->current_mode = NM_BASIC_MODE;

    // hide/show advanced/basic hboxes
    gtk_widget_hide (GTK_WIDGET (priv->advanced_hbox));
    gtk_widget_show_all (GTK_WIDGET (priv->basic_hbox));

    // Set the status of the toggle buttons
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->check_basic_usb), 
                                 nm_usb_setup_is_enabled(NM_USB_SETUP(priv->usbsetup)));

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->check_basic_bt), 
                                 nm_bt_setup_is_enabled(NM_BT_SETUP(priv->btsetup)));

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->check_basic_wlan), 
                                 nm_wlan_setup_is_enabled(NM_WLAN_SETUP(priv->wlansetup)));
    
    // Set the focus to the last selected
    if (priv->selected_checkbox == 0) {
        gtk_widget_grab_focus(GTK_WIDGET(priv->check_basic_usb));
    }
    else if (priv->selected_checkbox == 1) {
        gtk_widget_grab_focus(GTK_WIDGET(priv->check_basic_bt));
    }
    else if (priv->selected_checkbox == 2) {
        gtk_widget_grab_focus(GTK_WIDGET(priv->check_basic_wlan));
    }
    

    // hide/show basic/advanced buttons
    gtk_widget_hide (GTK_WIDGET (priv->basic_button));
    gtk_widget_show (GTK_WIDGET (priv->advanced_button));

}

static gboolean
nm_main_dialog_usb_apply (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    gboolean is_loaded = nm_usb_utils_is_loaded ();
    gboolean is_toggled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_basic_usb));

    if (!is_loaded && is_toggled) {

        gchar *gw;
        priv->settings = nm_settings_new ();

        if (!nm_settings_start (priv->settings)) {

            hildon_banner_show_information(NULL, NULL,
                                            "Could not read config file!\n\
                                             Please, open network manager and cofigure it.");
            return FALSE;
        }

        gw = nm_settings_get_value (priv->settings, NM_USB_SETTINGS_GROUP,
                                     NM_SETTINGS_GATEWAY);

        if (nm_usb_utils_load (gw)) {

            hildon_banner_show_information(NULL, NULL, "USB network enabled");
        } else {

            return FALSE;
        }
    } else if (is_loaded && !is_toggled) {

        if (nm_usb_utils_unload ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "USB file storage enabled");
        } else {

            return FALSE;
        }
    }

    return TRUE;
}

static gboolean
nm_main_dialog_bt_apply (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);
    NmBtUtils *bt_utils = nm_bt_utils_new ();

    gboolean is_loaded = (nm_bt_utils_radio_is_enabled (bt_utils) &&
                          nm_bt_utils_has_network_connection (bt_utils));
    gboolean is_toggled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_basic_bt));

    if (!is_loaded && is_toggled) {

        gchar *btaddr, *gw;
        priv->settings = nm_settings_new ();

        if (!nm_settings_start (priv->settings)) {

            hildon_banner_show_information(NULL, NULL,
                                            "Could not read config file!\n\
                                             Please, open network manager and cofigure it.");
            return FALSE;
        }

        btaddr = nm_settings_get_value (priv->settings, NM_BT_SETTINGS_GROUP,
                                         NM_SETTINGS_MAC);
        gw = nm_settings_get_value (priv->settings, NM_BT_SETTINGS_GROUP,
                                     NM_SETTINGS_GATEWAY);

        if (strcmp (btaddr, NM_BT_DEFAULT_MAC) == 0) {

            hildon_banner_show_information (NULL, NULL,
                                             "Please, open network manager and\nconfigure bluetooth interface!");
            return FALSE;
        } else if (nm_bt_utils_check_pand (bt_utils, btaddr) == NULL) {

            hildon_banner_show_information (NULL, NULL,
                                            "Unable to connect. Check if PAND is running on host!");
            return FALSE;
        }

        if (!nm_bt_utils_connect (bt_utils, btaddr, gw)) {

            hildon_banner_show_information (NULL, NULL,
                                            "Could not connect to PC");
            return FALSE;
        }

        hildon_banner_show_information(NULL, NULL, "Bluetooth network enabled");
    } else if (is_loaded && !is_toggled) {

        nm_bt_utils_remove_connection (bt_utils);

        hildon_banner_show_information(NULL, NULL,
                                        "Bluetooth network disabled");
    }

    g_object_unref (bt_utils);

    return TRUE;
}

static gboolean
nm_main_dialog_wlan_apply (NmMainDialog *self)
{
    NmMainDialogPrivate *priv = NM_MAIN_DIALOG_GET_PRIVATE (self);

    gboolean is_loaded =nm_wlan_utils_is_active ();
    gboolean is_toggled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->check_basic_wlan));

    if (!is_loaded && is_toggled) {

        if (nm_wlan_utils_activate ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "WLAN Ad-Hoc network enabled");
        } else {

            return FALSE;
        }
    } else if (is_loaded && !is_toggled) {

        if (nm_wlan_utils_deactivate ()) {

            hildon_banner_show_information(NULL, NULL,
                                            "WLAN Ad-Hoc network disabled");

        } else {

            return FALSE;
        }
    }

    return TRUE;
}
